//SPDX-FileCopyrightText: 2023 Yury Gubich <blue@macaw.me>
//SPDX-License-Identifier: GPL-3.0-or-later

#pragma once

#include <memory>
#include <thread>
#include <queue>
#include <chrono>
#include <functional>
#include <mutex>
#include <condition_variable>

#include "manager.h"
#include "utils/helpers.h"

namespace TM {
class Scheduler {
public:
    using Delay = std::chrono::milliseconds;
    using Task = std::function<void()>;

    Scheduler (std::weak_ptr<Manager> manager);
    ~Scheduler ();

    void start();
    void stop();
    void schedule(Task task, Delay delay);

private:
    void loop();

private:
    using Time = std::chrono::time_point<std::chrono::steady_clock>;
    using Record = std::pair<Time, Task>;

    std::weak_ptr<Manager> manager;
    std::priority_queue<
        Record,
        std::vector<Record>,
        FirstGreater<Record>
    > queue;
    std::mutex mutex;
    std::condition_variable cond;
    std::unique_ptr<std::thread> thread;
    bool running;
};
}
